@echo off
title Interesting Wins Search Tool V4.0 by Stefan Pohl, www.sp-cc.de
@echo ***************************************************************
@echo ***            Interesting Wins Search Tool V4.0            ***
@echo ***           (C) 2024, Stefan Pohl, www.sp-cc.de           ***
@echo ***************************************************************
@echo ***     This tool uses pgn-extract (C) David J. Barnes      ***
@echo ***************************************************************
@echo ***************************************************************

@echo Enter the name of your games pgn-file (without .pgn ending!) The file must be in the working folder!
set /p filename=
set gamebase=%filename%.pgn
@echo *********************************************************************************************
@echo Enter the maximum length of all non-draw games, which the tool has to investigate. For human games
@echo or adjucated engine games, at least 80 moves are recommended, for non-adjucated engine games, at
@echo least 90 or 100 is recommended. Values below 30 will be set to 100 and above 250 will be set to 250.
@echo Press RETURN, only, then 100 is chosen, too.
@echo off
set /p moves=
set /A movelimit=moves
if %movelimit% LSS 30 set /A movelimit=100
if %movelimit% GEQ 250 set /A movelimit=250
@echo *********************************************************************************************
@echo Enter the name of one engine/player, if you want wins of this engine getting filtered, only.
@echo Otherwise just press RETURN - all games will be investigated.
set engine=none
set /p engine=
set engine="%engine%"
@echo off

del interesting_wins.pgn 2>nul
del very_interesting_wins.pgn 2>nul

CALL :StartTimer
cd ./bin

del foundgames.pgn 2>nul
del foundtopgames.pgn 2>nul

@echo *********************************************************************************************
@echo *********************************************************************************************
@echo *********************************************************************************************
@echo|set /p="*** Filtering won games out of database, delete comments, check result-tags"

if %engine% == "none" (
  pgn-extract --quiet -bu%movelimit% --fixresulttags -Tr1-0 -Tr0-1 ../%gamebase% --output newsource.pgn > NUL
  goto no_engine
)
pgn-extract --quiet -Tw%engine% -bu%movelimit% --fixresulttags -Tr1-0 ../%gamebase% --output newsource.pgn > NUL
pgn-extract --quiet -Tb%engine% -bu%movelimit% --fixresulttags -Tr0-1 ../%gamebase% -anewsource.pgn > NUL
)
:no_engine
pgn-extract --quiet -Tr1-0 newsource.pgn --output whitewins.pgn > NUL
pgn-extract --quiet -Tr0-1 newsource.pgn --output blackwins.pgn > NUL
@echo ...done 
@echo *********************************************************************************************
@echo|set /p="*** Searching for Sacrifices: "

REM *** Search for the won games with sacrifices, played by the engine ***
@echo|set /p="1+ sacs white"
pgn-extract --quiet -y1_pawnsac_white whitewins.pgn --output results_opt1.pgn >NUL
del whitewins.pgn 2>nul
copy results_opt1.pgn whitewins.pgn > NUL
@echo|set /p="...1+ sacs black"
pgn-extract --quiet -y1_pawnsac_black blackwins.pgn --output blacksacs.pgn >NUL
REM *** Merge all found games in one single file ***
pgn-extract --quiet -aresults_opt1.pgn blacksacs.pgn >NUL

del blackwins.pgn 2>nul
copy blacksacs.pgn blackwins.pgn > NUL

@echo|set /p="...2-5+ sacs"
pgn-extract --quiet -y2_pawnsac_white whitewins.pgn --output results_opt2.pgn >NUL
pgn-extract --quiet -y2_pawnsac_black blackwins.pgn --output blacksacs.pgn >NUL
del whitewins.pgn 2>nul
del blackwins.pgn 2>nul
copy results_opt2.pgn whitewins.pgn > NUL
copy blacksacs.pgn blackwins.pgn > NUL
REM *** Merge all found games in one single file ***
pgn-extract --quiet -aresults_opt2.pgn blacksacs.pgn >NUL
fsutil file createnew blacksacs.pgn 0 >NUL

pgn-extract --quiet -y3_pawnsac_white whitewins.pgn --output results_opt3.pgn >NUL
pgn-extract --quiet -y3_pawnsac_black blackwins.pgn --output blacksacs.pgn >NUL
del whitewins.pgn 2>nul
del blackwins.pgn 2>nul
copy results_opt3.pgn whitewins.pgn > NUL
copy blacksacs.pgn blackwins.pgn > NUL
REM *** Merge all found games in one single file ***
pgn-extract --quiet -aresults_opt3.pgn blacksacs.pgn >NUL
fsutil file createnew blacksacs.pgn 0 >NUL

pgn-extract --quiet -y4_pawnsac_white whitewins.pgn --output results_opt4.pgn >NUL
pgn-extract --quiet -y4_pawnsac_black blackwins.pgn --output blacksacs.pgn >NUL
del whitewins.pgn 2>nul
del blackwins.pgn 2>nul
copy results_opt4.pgn whitewins.pgn > NUL
copy blacksacs.pgn blackwins.pgn > NUL
REM *** Merge all found games in one single file ***
pgn-extract --quiet -aresults_opt4.pgn blacksacs.pgn >NUL
fsutil file createnew blacksacs.pgn 0 >NUL

pgn-extract --quiet -y5_pawnsac_white whitewins.pgn --output results_opt5.pgn >NUL
pgn-extract --quiet -y5_pawnsac_black blackwins.pgn --output blacksacs.pgn >NUL
del whitewins.pgn 2>nul
del blackwins.pgn 2>nul
copy results_opt5.pgn whitewins.pgn > NUL
copy blacksacs.pgn blackwins.pgn > NUL
REM *** Merge all found games in one single file ***
pgn-extract --quiet -aresults_opt5.pgn blacksacs.pgn >NUL

pgn-extract --quiet -yqueensac_white whitewins.pgn --output results_opt9.pgn >NUL
pgn-extract --quiet -yqueensac_black blackwins.pgn --output blacksacs.pgn >NUL
REM *** Merge all found games in one single file ***
pgn-extract --quiet -aresults_opt9.pgn blacksacs.pgn >NUL

REM *** Delete games in the gamebases with lower sacs, which already occur in higher sacs files...***
pgn-extract --quiet -D results_opt9.pgn --output unique_opt9.pgn >NUL
pgn-extract --quiet -cresults_opt9.pgn -D -ounique_opt5.pgn results_opt5.pgn > NUL
pgn-extract --quiet -cresults_opt5.pgn -D -ounique_opt4.pgn results_opt4.pgn > NUL
pgn-extract --quiet -cresults_opt4.pgn -D -ounique_opt3.pgn results_opt3.pgn > NUL
pgn-extract --quiet -cresults_opt3.pgn -D -ounique_opt2.pgn results_opt2.pgn > NUL
pgn-extract --quiet -cresults_opt2.pgn -D -ounique_opt1.pgn results_opt1.pgn > NUL

REM *** Sort final databases by length ***
CALL :countgames unique_opt9.pgn
if %c_numb_pgn% GTR 0 (
  CALL :sortlength unique_opt9.pgn
  del unique_opt9.pgn 2>nul
  rename sortedlength.pgn unique_opt9.pgn
)
CALL :countgames unique_opt5.pgn
if %c_numb_pgn% GTR 0 (
  CALL :sortlength unique_opt5.pgn
  del unique_opt5.pgn 2>nul
  rename sortedlength.pgn unique_opt5.pgn
)
CALL :countgames unique_opt4.pgn
if %c_numb_pgn% GTR 0 (
  CALL :sortlength unique_opt4.pgn
  del unique_opt4.pgn 2>nul
  rename sortedlength.pgn unique_opt4.pgn
)
CALL :countgames unique_opt3.pgn
if %c_numb_pgn% GTR 0 (
  CALL :sortlength unique_opt3.pgn
  del unique_opt3.pgn 2>nul
  rename sortedlength.pgn unique_opt3.pgn
)
CALL :countgames unique_opt2.pgn
if %c_numb_pgn% GTR 0 (
  CALL :sortlength unique_opt2.pgn
  del unique_opt2.pgn 2>nul
  rename sortedlength.pgn unique_opt2.pgn
)
CALL :countgames unique_opt1.pgn
if %c_numb_pgn% GTR 0 (
  CALL :sortlength unique_opt1.pgn
  del unique_opt1.pgn 2>nul
  rename sortedlength.pgn unique_opt1.pgn
)

REM *** Adding the Annotator-Tag with the Sac-information and merge found games in output-file
del out9.pgn 2>nul

del newtag 2>nul
copy anno_9sac newtag > NUL
tagCreate newtag unique_opt9.pgn  >NUL
pgn-extract --quiet out9.pgn --output foundgames.pgn > NUL
pgn-extract --quiet out9.pgn --output foundtopgames.pgn > NUL

del newtag 2>nul
copy anno_5sac newtag > NUL
tagCreate newtag unique_opt5.pgn  >NUL
pgn-extract --quiet out9.pgn -afoundgames.pgn > NUL
pgn-extract --quiet out9.pgn -afoundtopgames.pgn > NUL

del newtag 2>nul
copy anno_4sac newtag > NUL
tagCreate newtag unique_opt4.pgn  >NUL
pgn-extract --quiet out9.pgn -afoundgames.pgn > NUL
pgn-extract --quiet out9.pgn -afoundtopgames.pgn > NUL

del newtag 2>nul
copy anno_3sac newtag > NUL
tagCreate newtag unique_opt3.pgn  >NUL
pgn-extract --quiet out9.pgn -afoundgames.pgn > NUL
pgn-extract --quiet out9.pgn -afoundtopgames.pgn > NUL

del newtag 2>nul
copy anno_2sac newtag > NUL
tagCreate newtag unique_opt2.pgn  >NUL
pgn-extract --quiet out9.pgn -afoundgames.pgn > NUL
pgn-extract --quiet out9.pgn -afoundtopgames.pgn > NUL

del newtag 2>nul
copy anno_1sac newtag > NUL
tagCreate newtag unique_opt1.pgn  >NUL
pgn-extract --quiet out9.pgn -afoundgames.pgn > NUL

del out9.pgn 2>nul

@echo ...done
@echo *********************************************************************************************
@echo *** Searching for games, which ended before endgame (material) was reached

pgn-extract --quiet -zno_endgame newsource.pgn --output results.pgn > NUL
pgn-extract --quiet -cresults.pgn -D -ono_endgame_wins.pgn newsource.pgn > NUL

REM *** Sort final database by length ***
CALL :countgames no_endgame_wins.pgn
if %c_numb_pgn% GTR 0 (
  CALL :sortlength no_endgame_wins.pgn
  del no_endgame_wins.pgn 2>nul
  rename sortedlength.pgn no_endgame_wins.pgn
)

REM *** Adding the Annotator-Tag with and merge found games in output-file
del newtag 2>nul
copy anno_before_endgame newtag > NUL
tagCreate newtag no_endgame_wins.pgn  >NUL
pgn-extract --quiet out9.pgn -afoundgames.pgn > NUL
pgn-extract --quiet out9.pgn -afoundtopgames.pgn > NUL
del out9.pgn 2>nul

@echo *********************************************************************************************
@echo *** Searching for games, which had a material imbalance (rook vs bishop and 2 pawns or so)

pgn-extract --quiet -zimbalance newsource.pgn --output imbalance.pgn > NUL

REM *** Sort final database by length ***
CALL :countgames imbalance.pgn
if %c_numb_pgn% GTR 0 (
  CALL :sortlength imbalance.pgn
  del imbalance.pgn 2>nul
  rename sortedlength.pgn imbalance.pgn
)

REM *** Adding the Annotator-Tag with and merge found games in output-file
del newtag 2>nul
copy anno_material_imbalance newtag > NUL
tagCreate newtag imbalance.pgn >NUL
pgn-extract --quiet out9.pgn -afoundgames.pgn > NUL

@echo *********************************************************************************************
@echo *** Deleting double games in the output-file and counting the remaining games
pgn-extract --quiet -D foundgames.pgn --output ../interesting_wins.pgn >NUL
pgn-extract --quiet -D foundtopgames.pgn --output ../very_interesting_wins.pgn >NUL

REM *** delete all working files ***
del out9.pgn 2>nul
del newtag 2>nul
del results.pgn 2>nul
del results_opt1.pgn 2>nul
del results_opt2.pgn 2>nul 
del results_opt3.pgn 2>nul 
del results_opt4.pgn 2>nul 
del results_opt5.pgn 2>nul 
del results_opt9.pgn 2>nul 
del unique_opt1.pgn 2>nul 
del unique_opt2.pgn 2>nul 
del unique_opt3.pgn 2>nul 
del unique_opt4.pgn 2>nul 
del unique_opt5.pgn 2>nul 
del unique_opt9.pgn 2>nul 
del whitewins.pgn 2>nul
del blackwins.pgn 2>nul
del blacksacs.pgn 2>nul
del no_endgame_wins.pgn 2>nul
del imbalance.pgn 2>nul
del foundgames.pgn 2>nul
del foundtopgames.pgn 2>nul
del newsource.pgn 2>nul
del sortedlength.pgn 2>nul

cd ../
CALL :countgames interesting_wins.pgn
@echo *********************************************************************************************
@echo *** Output-files are: interesting_wins.pgn, containing %c_numb_pgn% games and
CALL :countgames very_interesting_wins.pgn
@echo *** very_interesting_wins.pgn, containing %c_numb_pgn% games
@echo *********************************************************************************************
CALL :StopTimer
CALL :DisplayTimerResult
@echo *********************************************************************************************
:error
CALL ./bin/playwav %windir%/media/alarm02.wav
pause
goto :EOF

:countgames
set /A c_numb_pgn=0
for /f "tokens=2,* delims= " %%G in ('find /C "[White " %~1') do set /A c_numb_pgn=%%H
exit /B 0

:sortlength
del sortedlength.pgn 2>nul
pgn-extract --quiet -bu19 %~1 --output sortedlength.pgn > NUL
pgn-extract --quiet -bu29 -bl20 %~1 -asortedlength.pgn > NUL
pgn-extract --quiet -bu39 -bl30 %~1 -asortedlength.pgn > NUL
pgn-extract --quiet -bu49 -bl40 %~1 -asortedlength.pgn > NUL
pgn-extract --quiet -bu59 -bl50 %~1 -asortedlength.pgn > NUL
pgn-extract --quiet -bu69 -bl60 %~1 -asortedlength.pgn > NUL
pgn-extract --quiet -bu79 -bl70 %~1 -asortedlength.pgn > NUL
pgn-extract --quiet -bu89 -bl80 %~1 -asortedlength.pgn > NUL
pgn-extract --quiet -bu99 -bl90 %~1 -asortedlength.pgn > NUL
pgn-extract --quiet -bu109 -bl100 %~1 -asortedlength.pgn > NUL
pgn-extract --quiet -bu119 -bl110 %~1 -asortedlength.pgn > NUL
pgn-extract --quiet -bl120 %~1 -asortedlength.pgn > NUL
exit /B 0

:StartTimer
set StartTIME=%TIME%
for /f "usebackq tokens=1-4 delims=:., " %%f in (`echo %StartTIME: =0%`) do set /a Start100S=1%%f*360000+1%%g*6000+1%%h*100+1%%i-36610100
EXIT /B 0

:StopTimer
set StopTIME=%TIME%
for /f "usebackq tokens=1-4 delims=:., " %%f in (`echo %StopTIME: =0%`) do set /a Stop100S=1%%f*360000+1%%g*6000+1%%h*100+1%%i-36610100
if %Stop100S% LSS %Start100S% set /a Stop100S+=8640000
set /a TookTime=%Stop100S%-%Start100S%
set TookTimePadded=0%TookTime%
EXIT /B 0

:DisplayTimerResult
set /A t_elapsed=%TookTime:~0,-2%.%TookTimePadded:~-2% 2>nul
set /A t_seconds=%t_elapsed%
set /A t_minutes=%t_seconds%/60
set /A t_hours=%t_minutes%/60
set /A t_seconds-=%t_minutes%*60
set /A t_minutes-=%t_hours%*60
if %t_seconds% LEQ 9 set t_seconds=0%t_seconds%
if %t_minutes% LEQ 9 set t_minutes=0%t_minutes%
if %t_hours% LEQ 9 set t_hours=0%t_hours%
@echo *** Elapsed time: %t_hours%:%t_minutes%:%t_seconds%
EXIT /B 0




